@@ -52,7 +52,7 @@ The new version of the list, minus the deleted strings, is returned."
       (save-excursion
        (unwind-protect
 	   (let (list start s char)
-	     (setq work-buffer (generate-new-buffer "*vm-work*"))
+	     (setq work-buffer (vm-make-work-buffer))
 	     (set-buffer work-buffer)
 	     (insert string)
 	     (goto-char (point-min))
@@ -104,7 +104,7 @@ The new version of the list, minus the deleted strings, is returned."
 	     (if sepchar
 		 (setq nonspecials (concat nonspecials (list sepchar))
 		       sp+sepchar (concat "\t\f\n\r " (list sepchar))))
-	     (setq work-buffer (generate-new-buffer "*vm-work*"))
+	     (setq work-buffer (vm-make-work-buffer))
 	     (buffer-disable-undo work-buffer)
 	     (set-buffer work-buffer)
 	     (insert string)
@@ -182,8 +182,7 @@ The new version of the list, minus the deleted strings, is returned."
 	(let ((buffer-read-only nil))
 	  (insert string)))
     (let ((temp-buffer nil)
-	  (coding-system-for-read 'no-conversion)
-	  (coding-system-for-write 'no-conversion))
+	  (coding-system-for-write (vm-line-ending-coding-system)))
       (unwind-protect
 	  (save-excursion
 	    (setq temp-buffer (generate-new-buffer "*vm-work*"))
@@ -194,8 +193,9 @@ The new version of the list, minus the deleted strings, is returned."
 	    ;; writing out message separators
 	    (setq buffer-file-type nil)
 	    ;; Tell MULE to pick the correct newline conversion.
-	    (if (or vm-xemacs-mule-p vm-fsfemacs-mule-p)
-		(set-buffer-file-coding-system 'no-conversion nil))
+	    (if (fboundp 'set-buffer-file-coding-system)
+		(set-buffer-file-coding-system 
+		 (vm-line-ending-coding-system) nil))
 	    (write-region (point-min) (point-max) where t 'quiet))
 	(and temp-buffer (kill-buffer temp-buffer))))))
 
@@ -206,7 +206,10 @@ The new version of the list, minus the deleted strings, is returned."
 	 (while mp
 	   (vm-set-su-start-of (car mp) nil)
 	   (vm-set-su-end-of (car mp) nil)
-	   (setq mp (cdr mp))))))
+	   (setq mp (cdr mp)))))
+  (and (bufferp vm-folders-summary-buffer)
+       (null (buffer-name vm-folders-summary-buffer))
+       (setq vm-folders-summary-buffer nil)))
 
 (defun vm-check-for-killed-presentation ()
   (and (bufferp vm-presentation-buffer-handle)
@@ -285,6 +288,9 @@ The new version of the list, minus the deleted strings, is returned."
 (defun vm-delete-index-file-names (list)
   (vm-delete 'vm-index-file-name-p list))
 
+(defun vm-delete-directory-names (list)
+  (vm-delete 'file-directory-p list))
+
 (defun vm-index-file-name-p (file)
   (and (file-regular-p file)
        (stringp vm-index-file-suffix)
@@ -445,7 +451,7 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	(condition-case nil
 	    (unwind-protect
 		(save-excursion
-		  (setq temp-buffer (generate-new-buffer "*vm-work*"))
+		  (setq temp-buffer (vm-make-work-buffer))
 		  (set-buffer temp-buffer)
 		  (call-process "date" nil temp-buffer nil)
 		  (nth 4 (vm-parse (vm-buffer-string-no-properties)
@@ -521,45 +527,64 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
     (set-buffer target-buffer)))
 
 (if (not (fboundp 'vm-extent-property))
-    (if (fboundp 'overlay-get)
+    (if vm-fsfemacs-p
 	(fset 'vm-extent-property 'overlay-get)
       (fset 'vm-extent-property 'extent-property)))
 
+(if (not (fboundp 'vm-extent-object))
+    (if vm-fsfemacs-p
+	(fset 'vm-extent-object 'overlay-buffer)
+      (fset 'vm-extent-object 'extent-object)))
+
 (if (not (fboundp 'vm-set-extent-property))
-    (if (fboundp 'overlay-put)
+    (if vm-fsfemacs-p
 	(fset 'vm-set-extent-property 'overlay-put)
       (fset 'vm-set-extent-property 'set-extent-property)))
 
 (if (not (fboundp 'vm-set-extent-endpoints))
-    (if (fboundp 'move-overlay)
+    (if vm-fsfemacs-p
 	(fset 'vm-set-extent-endpoints 'move-overlay)
       (fset 'vm-set-extent-endpoints 'set-extent-endpoints)))
 
 (if (not (fboundp 'vm-make-extent))
-    (if (fboundp 'make-overlay)
+    (if vm-fsfemacs-p
 	(fset 'vm-make-extent 'make-overlay)
       (fset 'vm-make-extent 'make-extent)))
 
 (if (not (fboundp 'vm-extent-end-position))
-    (if (fboundp 'overlay-end)
+    (if vm-fsfemacs-p
 	(fset 'vm-extent-end-position 'overlay-end)
       (fset 'vm-extent-end-position 'extent-end-position)))
 
 (if (not (fboundp 'vm-extent-start-position))
-    (if (fboundp 'overlay-start)
+    (if vm-fsfemacs-p
 	(fset 'vm-extent-start-position 'overlay-start)
       (fset 'vm-extent-start-position 'extent-start-position)))
 
 (if (not (fboundp 'vm-detach-extent))
-    (if (fboundp 'delete-overlay)
+    (if vm-fsfemacs-p
 	(fset 'vm-detach-extent 'delete-overlay)
       (fset 'vm-detach-extent 'detach-extent)))
 
 (if (not (fboundp 'vm-extent-properties))
-    (if (fboundp 'overlay-properties)
+    (if vm-fsfemacs-p
 	(fset 'vm-extent-properties 'overlay-properties)
       (fset 'vm-extent-properties 'extent-properties)))
 
+(defun vm-extent-at (pos &optional object property)
+  (if (fboundp 'extent-at)
+      (extent-at pos object property)
+    (let ((o-list (overlays-at pos))
+	  (o nil))
+      (if (null property)
+	  (car o-list)
+	(while o-list
+	  (if (overlay-get (car o-list) property)
+	      (setq o (car o-list)
+		    o-list nil)
+	    (setq o-list (cdr o-list))))
+	o ))))
+
 (defun vm-copy-extent (e)
   (let ((props (vm-extent-properties e))
 	(ee (vm-make-extent (vm-extent-start-position e)
@@ -581,9 +606,27 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
     filename ))
 
 (defun vm-make-work-buffer (&optional name)
-  (let (work-buffer)
+  (let ((default-enable-multibyte-characters nil)
+	work-buffer)
     (setq work-buffer (generate-new-buffer (or name "*vm-workbuf*")))
     (buffer-disable-undo work-buffer)
+;; probably not worth doing since no one sets buffer-offer-save
+;; non-nil globally, do they?
+;;    (save-excursion
+;;      (set-buffer work-buffer)
+;;      (setq buffer-offer-save nil))
+    work-buffer ))
+
+(defun vm-make-multibyte-work-buffer (&optional name)
+  (let ((default-enable-multibyte-characters t)
+	work-buffer)
+    (setq work-buffer (generate-new-buffer (or name "*vm-workbuf*")))
+    (buffer-disable-undo work-buffer)
+;; probably not worth doing since no one sets buffer-offer-save
+;; non-nil globally, do they?
+;;    (save-excursion
+;;      (set-buffer work-buffer)
+;;      (setq buffer-offer-save nil))
     work-buffer ))
 
 (defun vm-insert-char (char &optional count ignored buffer)
@@ -624,7 +667,7 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
   (let ((work-buffer nil))
     (unwind-protect
 	(save-excursion
-	  (setq work-buffer (generate-new-buffer " *work*"))
+	  (setq work-buffer (vm-make-multibyte-work-buffer))
 	  (set-buffer work-buffer)
 	  (insert string)
 	  (funcall function)
@@ -642,6 +685,11 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	(setq list (cdr list))))
     (car list)))
 
+(defun vm-nonneg-string (n)
+  (if (< n 0)
+      "?"
+    (int-to-string n)))
+
 (defun vm-string-member (elt list)
   (let ((case-fold-search t)
 	(found nil)
@@ -720,3 +768,87 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	    (car time) (nth 1 time) (nth 2 time)
 	    (random 1000000)
 	    hostname)))
+
+(defun vm-keep-some-buffers (buffer ring-variable number-to-keep)
+  (if (memq buffer (symbol-value ring-variable))
+      (set ring-variable (delq buffer (symbol-value ring-variable))))
+  (set ring-variable (cons buffer (symbol-value ring-variable)))
+  (set ring-variable (vm-delete 'buffer-name
+				(symbol-value ring-variable) t))
+  (if (not (eq number-to-keep t))
+      (let ((extras (nthcdr (or number-to-keep 0)
+			    (symbol-value ring-variable))))
+	(mapcar (function
+		 (lambda (b)
+		   (and (buffer-name b)
+			(or (not (buffer-modified-p b))
+			    (not (vm-buffer-variable-value
+				  b buffer-offer-save)))
+			(kill-buffer b))))
+		extras)
+	(and (symbol-value ring-variable) extras
+	     (setcdr (memq (car extras) (symbol-value ring-variable))
+		     nil)))))
+
+(defvar enable-multibyte-characters)
+(defun vm-fsfemacs-nonmule-display-8bit-chars ()
+  (cond ((and vm-fsfemacs-p 
+	      (or (not vm-fsfemacs-mule-p)
+		  (and (boundp 'enable-multibyte-characters)
+		       (not enable-multibyte-characters))))
+	 (let* (tab (i 160))
+	   ;; We need the function make-display-table, but it is
+	   ;; in disp-table.el, which overwrites the value of
+	   ;; standard-display-table when it is loaded, which
+	   ;; sucks.  So here we cruftily copy just enough goop
+	   ;; out of disp-table.el so that a display table can be
+	   ;; created, and thereby avoid loading disp-table.
+	   (put 'display-table 'char-table-extra-slots 6)
+	   (setq tab (make-char-table 'display-table nil))
+	   (while (< i 256)
+	     (aset tab i (vector i))
+	     (setq i (1+ i)))
+	   (setq buffer-display-table tab)))))
+
+(defun vm-url-decode-string (string)
+  (vm-with-string-as-temp-buffer string 'vm-url-decode-buffer))
+
+(defun vm-url-decode-buffer ()
+  (let ((case-fold-search nil)
+	(hex-digit-alist '((?0 .  0)  (?1 .  1)  (?2 .  2)  (?3 .  3)
+			   (?4 .  4)  (?5 .  5)  (?6 .  6)  (?7 .  7)
+			   (?8 .  8)  (?9 .  9)  (?A . 10)  (?B . 11)
+			   (?C . 12)  (?D . 13)  (?E . 14)  (?F . 15)))
+	char)
+    (save-excursion
+      (goto-char (point-min))
+      (while (re-search-forward "%[0-9A-F][0-9A-F]" nil t)
+	(insert-char (+ (* (cdr (assq (char-after (- (point) 2))
+				      hex-digit-alist))
+			   16)
+			(cdr (assq (char-after (- (point) 1))
+				   hex-digit-alist)))
+		     1)
+	(delete-region (- (point) 1) (- (point) 4))))))
+
+(defun vm-md5-region (start end)
+  (if (fboundp 'md5)
+      (md5 (current-buffer) start end)
+    (let ((buffer nil)
+	  (curbuf (current-buffer)))
+      (unwind-protect
+	  (save-excursion
+	    (setq buffer (vm-make-work-buffer))
+	    (set-buffer buffer)
+	    (insert-buffer-substring curbuf start end)
+	    ;; call-process-region calls write-region.
+	    ;; don't let it do CR -> LF translation.
+	    (setq selective-display nil)
+	    (call-process-region (point-min) (point-max)
+				 (or shell-file-name "/bin/sh") t buffer nil
+				 shell-command-switch vm-pop-md5-program)
+	    ;; MD5 digest is 32 chars long
+	    ;; mddriver adds a newline to make neaten output for tty
+	    ;; viewing, make sure we leave it behind.
+	    (vm-buffer-substring-no-properties (point-min) (+ (point-min) 32)))
+	(and buffer (kill-buffer buffer))))))
