@@ -331,7 +331,11 @@ vm-included-text-prefix is prepended to every yanked line."
 	    (append-to-buffer b (vm-headers-of message)
 			      (vm-text-end-of message))
 	    (setq end (vm-marker (+ start (- (vm-text-end-of message)
-					     (vm-headers-of message))) b)))))
+					     (vm-headers-of message))) b))
+	    (if vm-display-using-mime
+		(progn
+		  (narrow-to-region start end)
+		  (vm-decode-mime-encoded-words))))))
       ;; get rid of read-only text properties on the text, as
       ;; they will only cause trouble.
       (let ((inhibit-read-only t))
@@ -368,24 +372,7 @@ as having been replied to, if appropriate."
 	   (vm-bury-buffer b)))))
 
 (defun vm-keep-mail-buffer (buffer)
-  ;; keep this buffer if the user demands it
-  (if (memq buffer vm-kept-mail-buffers)
-      (setq vm-kept-mail-buffers
-	    (delq buffer vm-kept-mail-buffers)))
-  (setq vm-kept-mail-buffers (cons buffer vm-kept-mail-buffers)
-	vm-kept-mail-buffers (vm-delete 'buffer-name
-					vm-kept-mail-buffers t))
-  (if (not (eq vm-keep-sent-messages t))
-      (let ((extras (nthcdr (or vm-keep-sent-messages 0)
-				vm-kept-mail-buffers)))
-	(mapcar (function
-		 (lambda (b)
-		   (and (buffer-name b)
-			(not (buffer-modified-p b))
-			(kill-buffer b))))
-		extras)
-	(and vm-kept-mail-buffers extras
-	     (setcdr (memq (car extras) vm-kept-mail-buffers) nil)))))
+  (vm-keep-some-buffers buffer 'vm-kept-mail-buffers vm-keep-sent-messages))
 
 (defun vm-help-tale ()
   (save-excursion
@@ -458,7 +445,9 @@ as having been replied to, if appropriate."
 			  (if (< timezone 0) "-" "+")
 			  (abs hour)
 			  (abs min))
-		  (format-time-string " (%Z)" time)
+;; localization in Europe and elsewhere can cause %Z to return
+;; 8-bit chars, which are forbidden in headers.
+;;		  (format-time-string " (%Z)" time)
 		  "\n"))))))
 
 (defun vm-mail-mode-remove-message-id-maybe ()
@@ -756,7 +745,7 @@ Subject: header manually."
 	(if miming
 	    (progn
 	      (setq mail-buffer (current-buffer))
-	      (set-buffer (generate-new-buffer "*vm-forward-buffer*"))
+	      (set-buffer (vm-make-work-buffer "*vm-forward-buffer*"))
 	      (setq header-end (point))
 	      (insert "\n"))
 	  (goto-char (point-min))
@@ -777,7 +766,11 @@ Subject: header manually."
 			(point)
 			(point-max))
 		       "\n")
-	       (insert "Content-Description: forwarded message\n"))
+	       (insert "Content-Description: forwarded message\n")
+	       ;; eight bit chars will get \201 prepended if we
+	       ;; don't do this.
+	       (if vm-fsfemacs-mule-p
+		   (set-buffer-multibyte t)))
 	      ((equal vm-forwarding-digest-type "rfc934")
 	       (vm-rfc934-encapsulate-messages
 		vm-forward-list vm-forwarded-headers
@@ -966,7 +959,7 @@ only marked messages will be put into the digest."
       (if miming
 	  (progn
 	    (setq mail-buffer (current-buffer))
-	    (set-buffer (generate-new-buffer "*vm-digest-buffer*"))
+	    (set-buffer (vm-make-work-buffer "*vm-digest-buffer*"))
 	    (setq header-end (point))
 	    (insert "\n")
 	    (setq start (point-marker)))
@@ -1092,6 +1085,7 @@ found, the current buffer remains selected."
 
 (defun vm-mail-to-mailto-url (url)
   (let ((address (car (vm-parse url "^mailto:\\(.+\\)"))))
+    (setq address (vm-url-decode-string address))
     (vm-select-folder-buffer)
     (vm-check-for-killed-summary)
     (vm-mail-internal nil address)
@@ -1389,7 +1383,8 @@ message."
 	  (or (vm-mail-mode-get-header-contents "From")
 	      (insert "From: " (user-login-name) "\n"))
 	  (or (vm-mail-mode-get-header-contents "Message-ID")
-	      (insert "Message-ID: <fake@fake.fake>\n"))
+	      (insert (format "Message-ID: <fake.%d.%d@fake.fake>\n"
+			      (random 1000000) (random 1000000))))
 	  (or (vm-mail-mode-get-header-contents "Date")
 	      (insert "Date: "
 		      (format-time-string "%a, %d %b %Y %H%M%S %Z"
@@ -1438,9 +1433,11 @@ message."
       (vm-continue-composing-message)
     (let ((buffer (vm-mail-internal
 		   (if to
-		       (format "message to %s" (vm-truncate-string to 20))
+		       (format "message to %s"
+			       (vm-truncate-roman-string to 20))
 		     nil)
 		   to subject)))
+      (goto-char (point-min))
       (re-search-forward (concat "^" mail-header-separator "$"))
       (beginning-of-line)
       (while other-headers
