@@ -21,13 +21,13 @@
 ;; it in the process due to buffer narrowing, and the fact that buffers are
 ;; indexed from 1 while strings are indexed from 0. :-(
 (defun vm-match-data ()
-  (let ((index '(9 8 7 6 5 4 3 2 1 0))
-        (list))
-    (while index
-      (setq list (cons (match-beginning (car index))
-		       (cons (match-end (car index)) list))
-	    index (cdr index)))
-    list ))
+  (let ((n (1- (/ (length (match-data)) 2)))
+        (list nil))
+    (while (>= n 0)
+      (setq list (cons (match-beginning n) 
+                       (cons (match-end n) list))
+            n (1- n)))
+    list))
 
 (defun vm-auto-select-folder (mp auto-folder-alist)
   (condition-case error-data
@@ -54,6 +54,8 @@
 			;; match data.
 			(save-excursion
 			  (set-buffer buf)
+			  (if vm-fsfemacs-mule-p
+			      (set-buffer-multibyte nil))
 			  (widen)
 			  (erase-buffer)
 			  (insert header)
@@ -327,6 +329,7 @@ The saved messages are flagged as `filed'."
 	   (if (null (vm-filed-flag m))
 	       (vm-set-filed-flag m t))
 	   (vm-increment count)
+	   (vm-modify-folder-totals folder 'saved 1 m)
 	   (vm-update-summary-and-mode-line)
 	   (setq mlist (cdr mlist)))))
       (if m
@@ -357,17 +360,18 @@ The saved messages are flagged as `filed'."
 	    (not (equal unexpanded-folder auto-folder)))
 	(setq vm-last-save-folder unexpanded-folder))
     (if vm-delete-after-saving
-	(vm-delete-message count))))
+	(vm-delete-message count))
+    folder ))
 
 (defun vm-save-message-sans-headers (file &optional count)
   "Save the current message to a file, without its header section.
-If the file already exists, the message will be appended to it.
-Prefix arg COUNT means save the next COUNT messages.  A negative COUNT means
-save the previous COUNT.
+If the file already exists, the message body will be appended to it.
+Prefix arg COUNT means save the next COUNT message bodiess.  A
+negative COUNT means save the previous COUNT bodies.
 
 When invoked on marked messages (via vm-next-command-uses-marks),
-all marked messages in the current folder are saved; other messages are
-ignored.
+only the next COUNT marked messages are saved; other intervening
+messages are ignored.
 
 The saved messages are flagged as `written'.
 
@@ -405,6 +409,9 @@ vm-save-message instead (normally bound to `s')."
 				 (find-file-noselect file))))
 	  ((and mlist vm-visit-when-saving)
 	   (setq file-buffer (vm-get-file-buffer file))))
+    (if (and (not (memq (vm-get-folder-type file) '(nil unknown)))
+	     (not (y-or-n-p "This file looks like a mail folder, append to it anyway? ")))
+	(error "Aborted"))
     (save-excursion
       (while mlist
 	(setq m (vm-real-message-of (car mlist)))
@@ -473,8 +480,9 @@ Output, if any, is displayed.  The message is not altered."
 	(mlist (if (eq last-command 'vm-next-command-uses-marks)
 		   (vm-select-marked-or-prefixed-messages 0)
 		 (list (car vm-message-pointer)))))
-    (set-buffer buffer)
-    (erase-buffer)
+    (save-excursion
+      (set-buffer buffer)
+      (erase-buffer))
     (while mlist
       (setq m (vm-real-message-of (car mlist)))
       (set-buffer (vm-buffer-of m))
@@ -491,17 +499,18 @@ Output, if any, is displayed.  The message is not altered."
 	      ((equal prefix-arg '(64))
 	       (narrow-to-region (vm-vheaders-of m) (vm-text-end-of m)))
 	      (t (narrow-to-region (point) (vm-text-end-of m))))
-	(let ((pop-up-windows (and pop-up-windows (eq vm-mutable-windows t))))
+	(let ((pop-up-windows (and pop-up-windows (eq vm-mutable-windows t)))
+	      ;; call-process-region calls write-region.
+	      ;; don't let it do CR -> LF translation.
+	      (selective-display nil))
 	  (call-process-region (point-min) (point-max)
 			       (or shell-file-name "sh")
 			       nil buffer nil shell-command-switch command)))
       (setq mlist (cdr mlist)))
-     (set-buffer buffer)
-     (if (not (zerop (buffer-size)))
-	 (vm-display buffer t '(vm-pipe-message-to-command)
-		     '(vm-pipe-message-to-command))
-       (vm-display nil nil '(vm-pipe-message-to-command)
-		   '(vm-pipe-message-to-command)))))
+    (vm-display nil nil '(vm-pipe-message-to-command)
+		'(vm-pipe-message-to-command))
+    (if (not (zerop (save-excursion (set-buffer buffer) (buffer-size))))
+	(display-buffer buffer))))
 
 (defun vm-print-message (&optional count)
   "Print the current message
@@ -535,8 +544,9 @@ Output, if any, is displayed.  The message is not altered."
 	 (m nil)
 	 (pop-up-windows (and pop-up-windows (eq vm-mutable-windows t)))
 	 (mlist (vm-select-marked-or-prefixed-messages count)))
-    (set-buffer buffer)
-    (erase-buffer)
+    (save-excursion
+      (set-buffer buffer)
+      (erase-buffer))
     (while mlist
       (setq m (vm-real-message-of (car mlist)))
       (set-buffer (vm-buffer-of m))
@@ -544,7 +554,7 @@ Output, if any, is displayed.  The message is not altered."
 	  (let ((work-buffer nil))
 	    (unwind-protect
 		(progn
-		  (setq work-buffer (generate-new-buffer "*vm-work*"))
+		  (setq work-buffer (vm-make-work-buffer))
 		  (set-buffer work-buffer)
 		  (vm-insert-region-from-buffer
 		   (vm-buffer-of m) (vm-vheaders-of m) (vm-text-of m))
@@ -557,7 +567,10 @@ Output, if any, is displayed.  The message is not altered."
 			(vm-mime-external-content-types-alist nil))
 		    (vm-decode-mime-layout (vm-mm-layout m)))
 		  (let ((pop-up-windows (and pop-up-windows
-					     (eq vm-mutable-windows t))))
+					     (eq vm-mutable-windows t)))
+			;; call-process-region calls write-region.
+			;; don't let it do CR -> LF translation.
+			(selective-display nil))
 		    (if need-tempfile
 			(write-region (point-min) (point-max)
 				      tempfile nil 0))
@@ -572,7 +585,10 @@ Output, if any, is displayed.  The message is not altered."
 	  (widen)
 	  (narrow-to-region (vm-vheaders-of m) (vm-text-end-of m))
 	  (let ((pop-up-windows (and pop-up-windows
-				     (eq vm-mutable-windows t))))
+				     (eq vm-mutable-windows t)))
+		;; call-process-region calls write-region.
+		;; don't let it do CR -> LF translation.
+		(selective-display nil))
 	    (if need-tempfile
 		(write-region (point-min) (point-max)
 			      tempfile nil 0))
@@ -583,9 +599,6 @@ Output, if any, is displayed.  The message is not altered."
 	    (if need-tempfile
 		(vm-error-free-call 'delete-file tempfile)))))
       (setq mlist (cdr mlist)))
-    (set-buffer buffer)
-    (if (not (zerop (buffer-size)))
-	(vm-display buffer t '(vm-pipe-message-to-command)
-		    '(vm-pipe-message-to-command))
-      (vm-display nil nil '(vm-pipe-message-to-command)
-		  '(vm-pipe-message-to-command)))))
+    (vm-display nil nil '(vm-print-message) '(vm-print-message))
+    (if (not (zerop (save-excursion (set-buffer buffer) (buffer-size))))
+	(display-buffer buffer))))
