@@ -17,6 +17,8 @@
 
 (provide 'vm-startup)
 
+(defvar enable-multibyte-characters)
+
 ;;;###autoload
 (defun vm (&optional folder read-only)
   "Read mail under Emacs.
@@ -35,8 +37,8 @@ be moved and appended to the resulting buffer.
 All the messages can be read by repeatedly pressing SPC.  Use `n'ext and
 `p'revious to move about in the folder.  Messages are marked for
 deletion with `d', and saved to another folder with `s'.  Quitting VM
-with `q' expunges deleted messages and saves the buffered folder to
-disk.
+with `q' saves the buffered folder to disk, but does not expunge
+deleted messages.  Use `###' to expunge deleted messages.
 
 See the documentation for vm-mode for more information."
   (interactive (list nil current-prefix-arg))
@@ -64,8 +66,11 @@ See the documentation for vm-mode for more information."
 				  default-directory))
 			    (inhibit-local-variables t)
 			    (enable-local-variables nil)
+			    ;; for Emacs/MULE
+			    (default-enable-multibyte-characters nil)
 			    ;; for XEmacs/Mule
-			    (coding-system-for-read 'no-conversion))
+			    (coding-system-for-read
+			         (vm-line-ending-coding-system)))
 			(message "Reading %s..." file)
 			(prog1 (find-file-noselect file)
 			  ;; update folder history
@@ -75,15 +80,20 @@ See the documentation for vm-mode for more information."
 				      (cons item vm-folder-history))))
 			  (message "Reading %s... done" file))))))))
       (set-buffer folder-buffer)
-      ;; for XEmacs/MULE
+      (if (and vm-fsfemacs-mule-p enable-multibyte-characters)
+	  (set-buffer-multibyte nil))
+      ;; for MULE
       ;;
       ;; If the file coding system is not a no-conversion variant,
-      ;; make it so by encoding all the text, then setting
-      ;; the file coding system and decoding it.
-      ;; This is only possible if a file is visited and then vm-mode
-      ;; is run on it afterwards.
+      ;; make it so by encoding all the text, then setting the
+      ;; file coding system and decoding it.  This situation is
+      ;; only possible if a file is visited and then vm-mode is
+      ;; run on it afterwards.
+      ;;
+      ;; There are separate code blocks for FSF Emacs and XEmacs
+      ;; because the coding systems have different names.
       (defvar buffer-file-coding-system)
-      (if (and vm-xemacs-mule-p
+      (if (and (or vm-xemacs-mule-p vm-xemacs-file-coding-p)
 	       (not (eq (get-coding-system buffer-file-coding-system)
 			(get-coding-system 'no-conversion-unix)))
 	       (not (eq (get-coding-system buffer-file-coding-system)
@@ -102,6 +112,27 @@ See the documentation for vm-mode for more information."
 		  (decode-coding-region (point-min) (point-max)
 					buffer-file-coding-system))
 	      (set-buffer-modified-p omodified))))
+      (if (and vm-fsfemacs-mule-p (null buffer-file-coding-system))
+	  (set-buffer-file-coding-system 'raw-text nil))
+      (if (and vm-fsfemacs-mule-p
+	       (not (eq (coding-system-base buffer-file-coding-system)
+			(coding-system-base 'raw-text-unix)))
+	       (not (eq (coding-system-base buffer-file-coding-system)
+			(coding-system-base 'raw-text-mac)))
+	       (not (eq (coding-system-base buffer-file-coding-system)
+			(coding-system-base 'raw-text-dos)))
+	       (not (eq (coding-system-base buffer-file-coding-system)
+			(coding-system-base 'no-conversion))))
+	  (let ((buffer-read-only nil)
+		(omodified (buffer-modified-p)))
+	    (unwind-protect
+		(progn
+		  (encode-coding-region (point-min) (point-max)
+					buffer-file-coding-system)
+		  (set-buffer-file-coding-system 'raw-text nil)
+		  (decode-coding-region (point-min) (point-max)
+					buffer-file-coding-system))
+	      (set-buffer-modified-p omodified))))
       (vm-check-for-killed-summary)
       (vm-check-for-killed-presentation)
       ;; If the buffer's not modified then we know that there can be no
@@ -134,6 +165,11 @@ See the documentation for vm-mode for more information."
 	    (buffer-disable-undo (current-buffer))
 	    (abbrev-mode 0)
 	    (auto-fill-mode 0)
+	    ;; If an 8-bit message arrives undeclared the 8-bit
+	    ;; characters in it should be displayed using the
+	    ;; user's default face charset, rather than as octal
+	    ;; escapes.
+	    (vm-fsfemacs-nonmule-display-8bit-chars)
 	    (vm-mode-internal)
 	    ;; If the buffer is modified we don't know if the
 	    ;; folder format has been changed to be different
@@ -142,13 +178,16 @@ See the documentation for vm-mode for more information."
 	    (if (not (buffer-modified-p))
 		(setq did-read-index-file (vm-read-index-file-maybe)))))
 
-      (vm-assimilate-new-messages nil (not did-read-index-file) nil)
+      ;; builds message list, reads attributes if they weren't
+      ;; read from an index file.
+      (vm-assimilate-new-messages nil (not did-read-index-file) nil t)
 
       (if (and first-time (not did-read-index-file))
 	  (progn
 	    (vm-gobble-visible-header-variables)
 	    (vm-gobble-bookmark)
 	    (vm-gobble-pop-retrieved)
+	    (vm-gobble-imap-retrieved)
 	    (vm-gobble-summary)
 	    (vm-gobble-labels)))
 
@@ -178,8 +217,11 @@ See the documentation for vm-mode for more information."
       ;; say this NOW, before the non-previewers read a message,
       ;; alter the new message count and confuse themselves.
       (if full-startup
-	  ;; save blurb so we can repeat it later as necessary.
-	  (setq totals-blurb (vm-emit-totals-blurb)))
+	  (progn
+	    ;; save blurb so we can repeat it later as necessary.
+	    (setq totals-blurb (vm-emit-totals-blurb))
+	    (and buffer-file-name
+		 (vm-store-folder-totals buffer-file-name (cdr vm-totals)))))
 
       (vm-thoughtfully-select-message)
       (vm-update-summary-and-mode-line)
@@ -200,7 +242,7 @@ See the documentation for vm-mode for more information."
 	    (if (and (vm-should-generate-summary)
 		     ;; don't generate a summary if recover-file is
 		     ;; likely to happen, since recover-file does
-		     ;; nothing useful in a summary buffer.
+		     ;; not work in a summary buffer.
 		     (not preserve-auto-save-file))
 		(vm-summarize t nil))
 	    ;; raise the summary frame if the user wants frames
@@ -222,7 +264,11 @@ See the documentation for vm-mode for more information."
 				      (current-buffer)))))))
 
       (if vm-message-list
-	  (vm-preview-current-message))
+	  ;; don't decode MIME if recover-file is
+	  ;; likely to happen, since recover-file does
+	  ;; not work in a presentation buffer.
+	  (let ((vm-auto-decode-mime-messages (not preserve-auto-save-file)))
+	    (vm-preview-current-message)))
 
       (run-hooks 'vm-visit-folder-hook)
 
@@ -243,7 +289,7 @@ See the documentation for vm-mode for more information."
 	       (not vm-folder-read-only))
 	  (progn
 	    (message "Checking for new mail for %s..."
-				(or buffer-file-name (buffer-name)))
+		     (or buffer-file-name (buffer-name)))
 	    (if (and (vm-get-spooled-mail t) (vm-assimilate-new-messages t))
 		(progn
 		  (setq totals-blurb (vm-emit-totals-blurb))
@@ -293,10 +339,11 @@ See the documentation for vm-mode for more information."
 (defun vm-mode (&optional read-only)
   "Major mode for reading mail.
 
-This is VM 6.53.
+This is VM 6.89.
 
 Commands:
    h - summarize folder contents
+   H - summarize contents of all folders
  C-t - toggle threads display
 
    n - go to next message
@@ -306,6 +353,7 @@ Commands:
  M-n - go to next unread message
  M-p - go to previous unread message
  RET - go to numbered message (uses prefix arg or prompts in minibuffer)
+ M-g - go to numbered message (uses prefix arg or prompts in minibuffer)
  TAB - go to last message seen
    ^ - go to parent of this message
  M-s - incremental search through the folder
@@ -315,11 +363,16 @@ Commands:
    b - scroll backward a page
    < - go to beginning of current message
    > - go to end of current message
+   [ - go to previous button
+   ] - go to next button
+   D - decode MIME if not already decoded.  If already decoded,
+       display all MIME objects as tags.  If already displaying
+       tags, show raw unecoded MIME>
 
    d - delete message, prefix arg deletes messages forward
  C-d - delete message, prefix arg deletes messages backward
    u - undelete
-   k - flag for deletion all messages with same subject as the current message
+   k - delete all messages with same subject as the current message
 
    r - reply (only to the sender of the message)
    R - reply with included text from the current message
@@ -349,7 +402,7 @@ Commands:
    w - write current message to a file without its headers (appends if exists)
    S - save entire folder to disk, does not expunge
    A - save unfiled messages to their vm-auto-folder-alist specified folders
-   # - expunge deleted messages (without saving folder)
+ ### - expunge deleted messages (without saving folder)
    q - quit VM, deleted messages are not expunged, folder is
        saved to disk if it is modified.  new messages are changed
        to be flagged as just unread.
@@ -374,7 +427,12 @@ Commands:
        M R - mark messages within the point/mark region in the summary
        M r - unmark messages within the point/mark region in the summary
        M V - toggle the marked-ness of all messages
-
+       M X - apply the selectors of a named virtual folder to the
+             messages in the current folder and mark all messages
+             that match those selectors.
+       M x - apply the selectors of a named virtual folder to the
+             messages in the current folder and unmark all messages
+             that match those selectors.
        M ? - partial help for mark commands
 
  W S - save the current window configuration to a name
@@ -406,6 +464,16 @@ Commands:
  l a - add labels to message
  l d - delete labels from message
 
+   $ - prefix for MIME commands.  Position the cursor over a MIME
+       tag and use these keystrokes to operate on a MIME object.
+
+       $ s - save the MIME object
+       $ p - print the MIME object
+       $ | - pipe the MIME object to a shell command.
+       $ RET - display the MIME object's text using the \"default\" face.
+       $ e - display the MIME object with an external viewer.
+       $ d - delete the MIME object from the message.
+
    L - reload your VM init file, ~/.vm
 
    % - change a folder to another type
@@ -440,6 +508,7 @@ Variables:
    vm-convert-folder-types
    vm-crash-box
    vm-crash-box-suffix
+   vm-default-From_-folder-type
    vm-default-folder-type
    vm-delete-after-archiving
    vm-delete-after-bursting
@@ -452,8 +521,13 @@ Variables:
    vm-display-buffer-hook
    vm-display-using-mime
    vm-edit-message-hook
+   vm-fill-paragraphs-containing-long-lines
+   vm-flush-interval
    vm-folder-directory
    vm-folder-read-only
+   vm-folders-summary-database
+   vm-folders-summary-directories
+   vm-folders-summary-format
    vm-follow-summary-cursor
    vm-forward-message-hook
    vm-forwarded-headers
@@ -464,12 +538,19 @@ Variables:
    vm-frame-per-composition
    vm-frame-per-edit
    vm-frame-per-folder
+   vm-frame-per-folders-summary
    vm-frame-per-help
    vm-frame-per-summary
    vm-highlighted-header-face
    vm-highlighted-header-regexp
    vm-honor-page-delimiters
    vm-image-directory
+   vm-imap-auto-expunge-alist
+   vm-imap-bytes-per-session
+   vm-imap-expunge-after-retrieving
+   vm-imap-max-message-size
+   vm-imap-messages-per-session
+   vm-imap-session-preauth-hook
    vm-index-file-suffix
    vm-in-reply-to-format
    vm-included-text-attribution-format
@@ -482,11 +563,13 @@ Variables:
    vm-jump-to-unread-messages
    vm-keep-crash-boxes
    vm-keep-sent-messages
+   vm-lynx-program
    vm-mail-check-interval
    vm-mail-header-from
    vm-mail-header-insert-date
    vm-mail-header-insert-message-id
    vm-mail-mode-hook
+   vm-mail-send-hook
    vm-make-crash-box-name
    vm-make-spool-file-name
    vm-mime-7bit-composition-charset
@@ -500,10 +583,15 @@ Variables:
    vm-mime-base64-decoder-switches
    vm-mime-base64-encoder-program
    vm-mime-base64-encoder-switches
-   vm-mime-button-format-alist
    vm-mime-button-face
+   vm-mime-button-format-alist
    vm-mime-charset-font-alist
+   vm-mime-confirm-delete
+   vm-mime-decode-for-preview
+   vm-mime-default-face-charset-exceptions
    vm-mime-default-face-charsets
+   vm-mime-delete-after-saving
+   vm-mime-delete-viewer-processes
    vm-mime-digest-discard-header-regexp
    vm-mime-digest-headers
    vm-mime-display-function
@@ -512,7 +600,13 @@ Variables:
    vm-mime-internal-content-type-exceptions
    vm-mime-internal-content-types
    vm-mime-max-message-size
+   vm-mime-qp-decoder-program
+   vm-mime-qp-decoder-switches
+   vm-mime-qp-encoder-program
+   vm-mime-qp-encoder-switches
    vm-mime-type-converter-alist
+   vm-mime-uuencode-decoder-program
+   vm-mime-uuencode-decoder-switches
    vm-mode-hook
    vm-mosaic-program
    vm-mosaic-program-switches
@@ -524,6 +618,8 @@ Variables:
    vm-mutable-windows
    vm-netscape-program
    vm-netscape-program-switches
+   vm-page-continuation-glyph
+   vm-paragraph-fill-column
    vm-pop-auto-expunge-alist
    vm-pop-bytes-per-session
    vm-pop-expunge-after-retrieving
@@ -536,6 +632,7 @@ Variables:
    vm-preview-read-messages
    vm-primary-inbox
    vm-quit-hook
+   vm-recognize-imap-maildrops
    vm-recognize-pop-maildrops
    vm-reply-hook
    vm-reply-ignored-addresses
@@ -562,8 +659,10 @@ Variables:
    vm-skip-read-messages
    vm-spool-file-suffixes
    vm-spool-files
+   vm-spooled-mail-waiting-hook
    vm-startup-with-summary
    vm-strip-reply-headers
+   vm-subject-significant-chars
    vm-summary-arrow
    vm-summary-format
    vm-summary-highlight-face
@@ -579,6 +678,7 @@ Variables:
    vm-undisplay-buffer-hook
    vm-unforwarded-header-regexp
    vm-url-browser
+   vm-url-retrieval-methods
    vm-url-search-limit
    vm-use-menus
    vm-use-toolbar
@@ -588,6 +688,7 @@ Variables:
    vm-visit-folder-hook
    vm-visit-when-saving
    vm-warp-mouse-to-new-frame
+   vm-wget-program
    vm-window-configuration-file
 "
   (interactive "P")
@@ -610,7 +711,8 @@ visited folder."
   (interactive
    (save-excursion
      (vm-session-initialization)
-     (vm-select-folder-buffer)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
 				default-directory))
@@ -626,7 +728,8 @@ visited folder."
 	      default-directory default nil nil 'vm-folder-history)
 	     current-prefix-arg))))
   (vm-session-initialization)
-  (vm-select-folder-buffer)
+  (vm-check-for-killed-folder)
+  (vm-select-folder-buffer-if-possible)
   (vm-check-for-killed-summary)
   (setq vm-last-visit-folder folder)
   (let ((default-directory (or vm-folder-directory default-directory)))
@@ -639,7 +742,8 @@ visited folder."
   (interactive
    (save-excursion
      (vm-session-initialization)
-     (vm-select-folder-buffer)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
 				default-directory))
@@ -654,6 +758,7 @@ visited folder."
 			""))
 	      default-directory default nil nil 'vm-folder-history)
 	     current-prefix-arg))))
+  (vm-session-initialization)
   (if (vm-multiple-frames-possible-p)
       (vm-goto-new-frame 'folder))
   (let ((vm-frame-per-folder nil)
@@ -668,7 +773,8 @@ visited folder."
   (interactive
    (save-excursion
      (vm-session-initialization)
-     (vm-select-folder-buffer)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
 				default-directory))
@@ -734,6 +840,7 @@ vm-visit-virtual-folder.")
 	      (funcall x (current-buffer))))
 	  (abbrev-mode 0)
 	  (auto-fill-mode 0)
+	  (vm-fsfemacs-nonmule-display-8bit-chars)
 	  (setq mode-name "VM Virtual"
 		mode-line-format vm-mode-line-format
 		buffer-read-only t
@@ -852,7 +959,8 @@ Optional argument TO is a string that should contain a comma separated
 recipient list."
   (interactive)
   (vm-session-initialization)
-  (vm-select-folder-buffer)
+  (vm-check-for-killed-folder)
+  (vm-select-folder-buffer-if-possible)
   (vm-check-for-killed-summary)
   (vm-mail-internal nil to)
   (run-hooks 'vm-mail-hook)
@@ -887,13 +995,93 @@ recipient list."
 	(vm-search-other-frames nil))
     (vm-mail to)))
 
+(fset 'vm-folders-summary-mode 'vm-mode)
+(put 'vm-folders-summary-mode 'mode-class 'special)
+
+;;;###autoload
+(defun vm-folders-summarize (&optional display raise)
+  "Generate a summary of the folders in your folder directories.
+Set `vm-folders-summary-directories' to specify the folder directories.
+Press RETURN or click mouse button 2 on an entry in the folders
+summary buffer to select a folder."
+  (interactive "p\np")
+  (vm-session-initialization)
+  (vm-check-for-killed-summary)
+  (if (not (featurep 'berkeley-db))
+      (error "Berkeley DB support needed to run this command"))
+  (if (null vm-folders-summary-database)
+      (error "'vm-folders-summary-database' must be non-nil to run this command"))
+  (if (null vm-folders-summary-buffer)
+      (let ((folder-buffer (and (eq major-mode 'vm-mode)
+				(current-buffer))))
+	(setq vm-folders-summary-buffer
+	      (let ((default-enable-multibyte-characters t))
+		(get-buffer-create "VM Folders Summary")))
+	(save-excursion
+	  (set-buffer vm-folders-summary-buffer)
+	  (abbrev-mode 0)
+	  (auto-fill-mode 0)
+	  (vm-fsfemacs-nonmule-display-8bit-chars)
+	  (if (fboundp 'buffer-disable-undo)
+	      (buffer-disable-undo (current-buffer))
+	    ;; obfuscation to make the v19 compiler not whine
+	    ;; about obsolete functions.
+	    (let ((x 'buffer-flush-undo))
+	      (funcall x (current-buffer))))
+	  (vm-folders-summary-mode-internal))
+	(vm-make-folders-summary-associative-hashes)
+	(vm-do-folders-summary)))
+  ;; if this command was run from a VM related buffer, select
+  ;; the folder buffer in the folders summary, but only if that
+  ;; folder has an entry there.
+  (and vm-mail-buffer
+       (vm-check-for-killed-folder))
+  (save-excursion
+    (and vm-mail-buffer
+	 (vm-select-folder-buffer))
+    (vm-check-for-killed-summary)
+    (let ((folder-buffer (and (eq major-mode 'vm-mode)
+			      (current-buffer)))
+	  fs )
+      (if (or (null vm-folders-summary-hash) (null folder-buffer)
+	      (null buffer-file-name))
+	  nil
+	(setq fs (symbol-value (intern-soft (vm-make-folders-summary-key
+					     buffer-file-name)
+					    vm-folders-summary-hash)))
+	(if (null fs)
+	    nil
+	  (vm-mark-for-folders-summary-update buffer-file-name)
+	  (set-buffer vm-folders-summary-buffer)
+	  (setq vm-mail-buffer folder-buffer)))))
+  (if display
+      (save-excursion
+	(vm-goto-new-folders-summary-frame-maybe)
+	(vm-display vm-folders-summary-buffer t
+		    '(vm-folders-summarize)
+		    (list this-command) (not raise))
+	;; need to do this after any frame creation because the
+	;; toolbar sets frame-specific height and width specifiers.
+	(set-buffer vm-folders-summary-buffer)
+	(and (vm-toolbar-support-possible-p) vm-use-toolbar
+	     (vm-toolbar-install-toolbar)))
+    (vm-display nil nil '(vm-folders-summarize)
+		(list this-command)))
+  (vm-update-summary-and-mode-line))
+
 ;;;###autoload
 (defun vm-submit-bug-report ()
   "Submit a bug report, with pertinent information to the VM bug list."
   (interactive)
   (require 'reporter)
-  ;; make sure the user doesn't try to use vm-mail here.
-  (let ((reporter-mailer '(mail)))
+  (require 'vm-version)
+  (require 'vm-vars)
+  ;; Use VM to send the bug report.  Could be trouble if vm-mail
+  ;; is what the user wants to complain about.  But most of the
+  ;; time we'll be fine and users like to use MIME to attach
+  ;; stuff to the reports.
+  (let ((reporter-mailer '(vm-mail))
+	(mail-user-agent 'vm-user-agent))
     (delete-other-windows)
     (reporter-submit-bug-report
      vm-maintainer-address
@@ -918,6 +1106,7 @@ recipient list."
       'vm-convert-folder-types
       'vm-crash-box
       'vm-crash-box-suffix
+      'vm-default-From_-folder-type
       'vm-default-folder-type
       'vm-delete-after-archiving
       'vm-delete-after-bursting
@@ -932,9 +1121,13 @@ recipient list."
       'vm-display-using-mime
       'vm-edit-message-hook
       'vm-edit-message-mode
+      'vm-fill-paragraphs-containing-long-lines
       'vm-flush-interval
       'vm-folder-directory
       'vm-folder-read-only
+      'vm-folders-summary-database
+      'vm-folders-summary-directories
+      'vm-folders-summary-format
       'vm-follow-summary-cursor
       'vm-forward-message-hook
       'vm-forwarded-headers
@@ -945,12 +1138,20 @@ recipient list."
       'vm-frame-per-composition
       'vm-frame-per-edit
       'vm-frame-per-folder
+      'vm-frame-per-folders-summary
       'vm-frame-per-help
       'vm-frame-per-summary
       'vm-highlight-url-face
       'vm-highlighted-header-regexp
       'vm-honor-page-delimiters
       'vm-image-directory
+;; IMAP passwords might be listed here
+;;      'vm-imap-auto-expunge-alist
+      'vm-imap-bytes-per-session
+      'vm-imap-expunge-after-retrieving
+      'vm-imap-max-message-size
+      'vm-imap-messages-per-session
+      'vm-imap-session-preauth-hook
       'vm-in-reply-to-format
       'vm-included-text-attribution-format
       'vm-included-text-discard-header-regexp
@@ -964,6 +1165,7 @@ recipient list."
       'vm-jump-to-unread-messages
       'vm-keep-crash-boxes
       'vm-keep-sent-messages
+      'vm-lynx-program
       'vm-mail-header-from
       'vm-mail-header-insert-date
       'vm-mail-header-insert-message-id
@@ -972,6 +1174,7 @@ recipient list."
       'vm-make-spool-file-name
       'vm-mail-check-interval
       'vm-mail-mode-hook
+      'vm-mail-send-hook
       'vm-mime-7bit-composition-charset
       'vm-mime-8bit-composition-charset
       'vm-mime-8bit-text-transfer-encoding
@@ -983,10 +1186,15 @@ recipient list."
       'vm-mime-base64-decoder-switches
       'vm-mime-base64-encoder-program
       'vm-mime-base64-encoder-switches
-      'vm-mime-button-format-alist
       'vm-mime-button-face
+      'vm-mime-button-format-alist
       'vm-mime-charset-font-alist
+      'vm-mime-confirm-delete
+      'vm-mime-decode-for-preview
+      'vm-mime-default-face-charset-exceptions
       'vm-mime-default-face-charsets
+      'vm-mime-delete-after-saving
+      'vm-mime-delete-viewer-processes
       'vm-mime-digest-discard-header-regexp
       'vm-mime-digest-headers
       'vm-mime-display-function
@@ -995,7 +1203,13 @@ recipient list."
       'vm-mime-internal-content-type-exceptions
       'vm-mime-internal-content-types
       'vm-mime-max-message-size
+      'vm-mime-qp-decoder-program
+      'vm-mime-qp-decoder-switches
+      'vm-mime-qp-encoder-program
+      'vm-mime-qp-encoder-switches
       'vm-mime-type-converter-alist
+      'vm-mime-uuencode-decoder-program
+      'vm-mime-uuencode-decoder-switches
       'vm-mode-hook
       'vm-mode-hooks
       'vm-mosaic-program
@@ -1009,6 +1223,8 @@ recipient list."
       'vm-mutable-windows
       'vm-netscape-program
       'vm-netscape-program-switches
+      'vm-page-continuation-glyph
+      'vm-paragraph-fill-column
 ;; POP passwords might be listed here
 ;;      'vm-pop-auto-expunge-alist
       'vm-pop-bytes-per-session
@@ -1022,6 +1238,7 @@ recipient list."
       'vm-preview-read-messages
       'vm-primary-inbox
       'vm-quit-hook
+      'vm-recognize-imap-maildrops
       'vm-recognize-pop-maildrops
       'vm-reply-hook
 ;; don't feed the spammers or crackers
@@ -1050,8 +1267,10 @@ recipient list."
 ;; don't send vm-spool-files by default, might contain passwords
 ;;      'vm-spool-files
       'vm-spool-file-suffixes
+      'vm-spooled-mail-waiting-hook
       'vm-startup-with-summary
       'vm-strip-reply-headers
+      'vm-subject-significant-chars
       'vm-summary-format
       'vm-summary-highlight-face
       'vm-summary-mode-hook
@@ -1069,6 +1288,7 @@ recipient list."
       'vm-undisplay-buffer-hook
       'vm-unforwarded-header-regexp
       'vm-url-browser
+      'vm-url-retrieval-methods
       'vm-url-search-limit
       'vm-use-menus
       'vm-use-toolbar
@@ -1078,19 +1298,20 @@ recipient list."
       'vm-visit-folder-hook
       'vm-visit-when-saving
       'vm-warp-mouse-to-new-frame
+      'vm-wget-program
       'vm-window-configuration-file
 ;; see what the user had loaded
       'features
       )
      nil
      nil
-     "Please change the Subject header to a concise bug description.\nRemember to cover the basics, that is, what you expected to\nhappen and what in fact did happen.  Please remove these\ninstructions from your message.")
+     "Please change the Subject header to a concise bug description.\nIn this report, remember to cover the basics, that is, what you expected to\nhappen and what in fact did happen.  Please remove these\ninstructions from your message.")
     (save-excursion
       (goto-char (point-min))
-      (mail-position-on-field "Subject")
-      (beginning-of-line)
-      (delete-region (point) (progn (forward-line) (point)))
-      (insert "Subject: VM " vm-version " induces a brain tumor in the user.\n         It is the tumor that creates the hallucinations.\n"))))
+      (mail-position-on-field "Subject"))))
+;;      (beginning-of-line)
+;;      (delete-region (point) (progn (forward-line) (point)))
+;;      (insert "Subject: VM " vm-version " induces a brain tumor in the user.\n         It is the tumor that creates the hallucinations.\n"))))
 
 (defun vm-load-init-file (&optional interactive)
   (interactive "p")
@@ -1114,9 +1335,7 @@ recipient list."
 		  (and (= emacs-major-version 19)
 		       (< emacs-minor-version 34))))
 	 (error "VM %s must be run on Emacs 19.34 or a later v19 version."
-		vm-version))
-	((and vm-fsfemacs-p (= emacs-major-version 20))
-	 (error "VM has not been ported to v20 Emacs.  Running VM in this environment is not advised."))))
+		vm-version))))
 
 (defun vm-set-debug-flags ()
   (or stack-trace-on-error
@@ -1155,8 +1374,14 @@ recipient list."
 		 (not (facep 'gui-button-face)))
 	    (progn
 	      (make-face 'gui-button-face)
-	      (set-face-foreground 'gui-button-face "black")
-	      (set-face-background 'gui-button-face "gray75")))
+	      (cond ((eq window-system 'x)
+		     (set-face-foreground 'gui-button-face "black")
+		     (set-face-background 'gui-button-face "gray75"))
+		    (t
+		     ;; use primary color names, since fancier
+		     ;; names may not be valid.
+		     (set-face-foreground 'gui-button-face "white")
+		     (set-face-background 'gui-button-face "red")))))
 	(and (vm-mouse-support-possible-p)
 	     (vm-mouse-install-mouse))
 	(and (vm-menu-support-possible-p)
