@@ -1,5 +1,5 @@
 ;;; Summary gathering and formatting routines for VM
-;;; Copyright (C) 1989-1995 Kyle E. Jones
+;;; Copyright (C) 1989-1995, 2000 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -63,11 +63,13 @@ mandatory."
       (let ((b (current-buffer))
 	    (read-only vm-folder-read-only))
 	(setq vm-summary-buffer
-	      (get-buffer-create (format "%s Summary" (buffer-name))))
+	      (let ((default-enable-multibyte-characters t))
+		(get-buffer-create (format "%s Summary" (buffer-name)))))
 	(save-excursion
 	  (set-buffer vm-summary-buffer)
 	  (abbrev-mode 0)
 	  (auto-fill-mode 0)
+	  (vm-fsfemacs-nonmule-display-8bit-chars)
 	  (if (fboundp 'buffer-disable-undo)
 	      (buffer-disable-undo (current-buffer))
 	    ;; obfuscation to make the v19 compiler not whine
@@ -286,25 +288,35 @@ mandatory."
 	    (and old-window (select-window old-window)))))))
 
 (defun vm-summary-highlight-region (start end face)
-  (cond (vm-fsfemacs-p
-	 (if (and vm-summary-overlay (overlay-buffer vm-summary-overlay))
-	     (move-overlay vm-summary-overlay start end)
-	   (setq vm-summary-overlay (make-overlay start end))
-	   (overlay-put vm-summary-overlay 'evaporate nil)
-	   (overlay-put vm-summary-overlay 'face face)))
-	(vm-xemacs-p
-	 (if (and vm-summary-overlay (extent-end-position vm-summary-overlay))
-	     (set-extent-endpoints vm-summary-overlay start end)
-	   (setq vm-summary-overlay (make-extent start end))
-	   ;; the reason this isn't needed under FSF Emacs is
-	   ;; that insert-before-markers also inserts before
-	   ;; overlays!  so a summary update of an entry just
-	   ;; before this overlay in the summary buffer won't
-	   ;; leak into the overlay, but it _will_ leak into an
-	   ;; XEmacs extent.
-	   (set-extent-property vm-summary-overlay 'start-open t)
-	   (set-extent-property vm-summary-overlay 'detachable nil)
-	   (set-extent-property vm-summary-overlay 'face face)))))
+  (vm-summary-xxxx-highlight-region start end face 'vm-summary-overlay))
+
+(defun vm-folders-summary-highlight-region (start end face)
+  (vm-summary-xxxx-highlight-region start end face
+				    'vm-folders-summary-overlay))
+
+(defun vm-summary-xxxx-highlight-region (start end face var)
+  (let ((ooo (symbol-value var)))
+    (cond (vm-fsfemacs-p
+	   (if (and ooo (overlay-buffer ooo))
+	       (move-overlay ooo start end)
+	     (setq ooo (make-overlay start end))
+	     (set var ooo)
+	     (overlay-put ooo 'evaporate nil)
+	     (overlay-put ooo 'face face)))
+	  (vm-xemacs-p
+	   (if (and ooo (extent-end-position ooo))
+	       (set-extent-endpoints ooo start end)
+	     (setq ooo (make-extent start end))
+	     (set var ooo)
+	     ;; the reason this isn't needed under FSF Emacs is
+	     ;; that insert-before-markers also inserts before
+	     ;; overlays!  so a summary update of an entry just
+	     ;; before this overlay in the summary buffer won't
+	     ;; leak into the overlay, but it _will_ leak into an
+	     ;; XEmacs extent.
+	     (set-extent-property ooo 'start-open t)
+	     (set-extent-property ooo 'detachable nil)
+	     (set-extent-property ooo 'face face))))))
 
 (defun vm-auto-center-summary ()
   (if vm-auto-center-summary
@@ -549,41 +561,72 @@ mandatory."
 	      (setq contents (vm-matched-header-contents))))))
       contents )))
 
+;; Do not use Emacs 20's string-width here.
+;; It does not consider buffer-display-table.
+(defun vm-string-width (string)
+  (if (not (fboundp 'char-width))
+      (length string)
+    (let ((i 0)
+	  (lim (length string))
+	  (total 0))
+      (while (< i lim)
+	(setq total (+ total (char-width (aref string i)))
+	      i (1+ i)))
+      total )))
+
 (defun vm-left-justify-string (string width)
-  (if (>= (length string) width)
-      string
-    (concat string (make-string (- width (length string)) ?\ ))))
+  (let ((sw (vm-string-width string)))
+    (if (>= sw width)
+	string
+      (concat string (make-string (- width sw) ?\ )))))
 
 (defun vm-right-justify-string (string width)
-  (if (>= (length string) width)
-      string
-    (concat (make-string (- width (length string)) ?\ ) string)))
+  (let ((sw (vm-string-width string)))
+    (if (>= sw width)
+	string
+      (concat (make-string (- width sw) ?\ ) string))))
 
+;; I don't think number glyphs ever have a width > 1
 (defun vm-numeric-left-justify-string (string width)
-  (if (>= (length string) width)
-      string
-    (concat string (make-string (- width (length string)) ?0))))
+  (let ((sw (length string)))
+    (if (>= sw width)
+	string
+      (concat string (make-string (- width sw) ?0)))))
 
+;; I don't think number glyphs ever have a width > 1
 (defun vm-numeric-right-justify-string (string width)
-  (if (>= (length string) width)
-      string
-    (concat (make-string (- width (length string)) ?0) string)))
+  (let ((sw (length string)))
+    (if (>= sw width)
+	string
+      (concat (make-string (- width sw) ?0) string))))
 
 (defun vm-truncate-string (string width)
-  (cond
-;; doesn't work because the width of wide chars such as the Kanji
-;; glyphs as not even multiples of the default face's font width.
-;;	((fboundp 'char-width)
-;;	 (let ((i 0)
-;;	       (lim (length string))
-;;	       (total 0))
-;;	   (while (and (< i lim) (<= total width))
-;;	     (setq total (+ total (char-width (aref string i)))
-;;		   i (1+ i)))
-;;	   (if (<= total width)
-;;	       string
-;;	     (substring string 0 (1- i)))))
-	((<= (length string) (vm-abs width))
+  (cond ((fboundp 'char-width)
+	 (cond ((> width 0)
+		(let ((i 0)
+		      (lim (length string))
+		      (total 0))
+		  (while (and (< i lim) (< total width))
+		    (setq total (+ total (char-width (aref string i)))
+			  i (1+ i)))
+		  (if (< total width)
+		      string
+		    (substring string 0 i))))
+	       (t
+		(let ((i (1- (length string)))
+		      (lim -1)
+		      (total 0))
+		  (setq width (- width))
+		  (while (and (> i lim) (< total width))
+		    (setq total (+ total (char-width (aref string i)))
+			  i (1- i)))
+		  (if (< total width)
+		      string
+		    (substring string (1+ i)))))))
+	(t (vm-truncate-roman-string string width))))
+
+(defun vm-truncate-roman-string (string width)
+  (cond ((<= (length string) (vm-abs width))
 	 string)
 	((< width 0)
 	 (substring string width))
@@ -713,7 +756,11 @@ mandatory."
       (cond ((string-match "\\`[4-9]." string)
 	     ;; Assume that any two digits less than 40 are a date and not
 	     ;; a year.  The world will surely end soon.
-	     (setq year (concat "19" string)))
+	     (setq year (concat "19" string))
+	     ;; assume any 2-digit year before 1970 is really a date past
+	     ;; the year 2000.
+	     (if (< (string-to-int year) 1970)
+		 (setq year (int-to-string (+ 100 (string-to-int year))))))
 	    ((< (length string) 3)
 	     (setq monthday string))
 	    (t (setq year string))))
@@ -1034,6 +1081,7 @@ mandatory."
     (while mp
       (vm-set-summary-of (car mp) nil)
       (vm-mark-for-summary-update (car mp))
+      (vm-set-modflag-of (car mp) t)
       (setq mp (cdr mp)))
     (vm-stuff-folder-attributes nil)
     (set-buffer-modified-p t)
@@ -1075,4 +1123,509 @@ mandatory."
 	  (buffer-string))
       (and work-buffer (kill-buffer work-buffer)))))
 
-				 
+(defun vm-make-folder-summary ()
+  (make-vector vm-folder-summary-vector-length nil))
+
+(defun vm-fs-folder-of (fs) (aref fs 0))
+(defun vm-fs-total-count-of (fs) (aref fs 1))
+(defun vm-fs-new-count-of (fs) (aref fs 2))
+(defun vm-fs-unread-count-of (fs) (aref fs 3))
+(defun vm-fs-deleted-count-of (fs) (aref fs 4))
+(defun vm-fs-start-of (fs) (aref fs 5))
+(defun vm-fs-end-of (fs) (aref fs 6))
+(defun vm-fs-folder-key-of (fs) (aref fs 7))
+(defun vm-fs-mouse-track-overlay-of (fs) (aref fs 8))
+(defun vm-fs-short-folder-of (fs) (aref fs 9))
+(defun vm-fs-modflag-of (fs) (aref fs 10))
+
+(defun vm-set-fs-folder-of (fs x) (aset fs 0 x))
+(defun vm-set-fs-total-count-of (fs x) (aset fs 1 x))
+(defun vm-set-fs-new-count-of (fs x) (aset fs 2 x))
+(defun vm-set-fs-unread-count-of (fs x) (aset fs 3 x))
+(defun vm-set-fs-deleted-count-of (fs x) (aset fs 4 x))
+(defun vm-set-fs-start-of (fs x) (aset fs 5 x))
+(defun vm-set-fs-end-of (fs x) (aset fs 6 x))
+(defun vm-set-fs-folder-key-of (fs x) (aset fs 7 x))
+(defun vm-set-fs-mouse-track-overlay-of (fs x) (aset fs 8 x))
+(defun vm-set-fs-short-folder-of (fs x) (aset fs 9 x))
+(defun vm-set-fs-modflag-of (fs x) (aset fs 10 x))
+
+(defun vm-fs-spooled (fs)
+  (let ((count 0)
+	(list (symbol-value
+	       (intern-soft (vm-fs-folder-key-of fs)
+			    vm-folders-summary-folder-hash))))
+    (while list
+      (setq count (+ count (car (vm-get-folder-totals (car list))))
+	    list (cdr list)))
+    (int-to-string count)))
+
+(defun vm-make-folders-summary-key (folder &optional dir)
+  (cond ((and (stringp vm-recognize-pop-maildrops)
+	      (string-match vm-recognize-pop-maildrops folder))
+	 (vm-safe-popdrop-string folder))
+	((and (stringp vm-recognize-imap-maildrops)
+	      (string-match vm-recognize-imap-maildrops folder))
+	 (vm-safe-imapdrop-string folder))
+	(t
+	 (concat "folder-summary0:"
+		 (file-truename
+		  (expand-file-name folder (or dir vm-folder-directory)))))))
+
+(defun vm-open-folders-summary-database (mode)
+  (condition-case data
+      (open-database vm-folders-summary-database 'berkeley-db 'hash mode)
+    (error (message "open-database signaled: %S" data)
+	   (sleep-for 2)
+	   nil )))
+
+(defun vm-get-folder-totals (folder)
+  (let ((default "(0 0 0 0)") fs db key data)
+    (catch 'done
+      (if (null vm-folders-summary-database)
+	  (throw 'done (read default)))
+      (if (not (featurep 'berkeley-db))
+	  (throw 'done (read default)))
+      (if (null (setq db (vm-open-folders-summary-database "rw+")))
+	  (throw 'done (read default)))
+      (setq key (vm-make-folders-summary-key folder)
+	    data (read (get-database key db default)))
+      (close-database db)
+      data )))
+
+(defun vm-store-folder-totals (folder totals)
+  (let (fs db key data)
+    (catch 'done
+      (if (null vm-folders-summary-database)
+	  (throw 'done nil))
+      (if (not (featurep 'berkeley-db))
+	  (throw 'done nil))
+      (if (null (setq db (vm-open-folders-summary-database "rw+")))
+	  (throw 'done nil))
+      (setq key (vm-make-folders-summary-key folder)
+	    data (prin1-to-string totals))
+      (put-database key data db t)
+      (close-database db)
+      (if (null vm-folders-summary-hash)
+	  nil
+	(setq fs (intern-soft key vm-folders-summary-hash)
+	      fs (symbol-value fs))
+	(if (null fs)
+	    nil
+	  (vm-set-fs-total-count-of fs (int-to-string (car totals)))
+	  (vm-set-fs-new-count-of fs (int-to-string (nth 1 totals)))
+	  (vm-set-fs-unread-count-of fs (int-to-string (nth 2 totals)))
+	  (vm-set-fs-deleted-count-of fs (int-to-string (nth 3 totals)))))
+      (vm-mark-for-folders-summary-update folder))))
+
+(defun vm-modify-folder-totals (folder action &rest objects)
+  (let (fs db totals key data)
+    (catch 'done
+      (if (null vm-folders-summary-database)
+	  (throw 'done nil))
+      (if (not (featurep 'berkeley-db))
+	  (throw 'done nil))
+      (if (null (setq db (vm-open-folders-summary-database "r")))
+	  (throw 'done nil))
+      (setq key (vm-make-folders-summary-key folder))
+      (setq totals (get-database key db))
+      (close-database db)
+      (if (null totals)
+	  (throw 'done nil))
+      (setq totals (read totals))
+      (cond ((eq action 'arrived)
+	     (let ((arrived (car objects)) c n)
+	       (setcar totals (+ (car totals) arrived))
+	       (setq c (cdr totals))
+	       (setcar c (+ (car c) arrived))))
+	    ((eq action 'saved)
+	     (let ((arrived (car objects))
+		   (m (nth 1 objects)) c n)
+	       (setcar totals (+ (car totals) arrived))
+	       ;; increment new and unread counts if necessary.
+	       ;; messages are never saved with the deleted flag
+	       ;; set no need to check that.
+	       (setq c (cdr totals))
+	       (if (eq (car c) -1)
+		   nil
+		 (if (vm-new-flag m)
+		     (setcar c (+ (car c) arrived))))
+	       (setq c (cdr c))
+	       (if (eq (car c) -1)
+		   nil
+		 (if (vm-unread-flag m)
+		     (setcar c (+ (car c) arrived)))))))
+      (setq data (prin1-to-string totals))
+      (if (null (setq db (vm-open-folders-summary-database "rw+")))
+	  (throw 'done nil))
+      (put-database key data db t)
+      (close-database db)
+      (if (null vm-folders-summary-hash)
+	  nil
+	(setq fs (intern-soft key vm-folders-summary-hash)
+	      fs (symbol-value fs))
+	(if (null fs)
+	    nil
+	  (vm-set-fs-total-count-of fs (int-to-string (car totals)))
+	  (vm-set-fs-new-count-of fs (int-to-string (nth 1 totals)))
+	  (vm-set-fs-unread-count-of fs (int-to-string (nth 2 totals)))
+	  (vm-set-fs-deleted-count-of fs (int-to-string (nth 3 totals)))))
+      (vm-mark-for-folders-summary-update folder))))
+
+(defun vm-folders-summary-sprintf (format layout)
+  ;; compile the format into an eval'able s-expression
+  ;; if it hasn't been compiled already.
+  (let ((match (assoc format vm-folders-summary-compiled-format-alist)))
+    (if (null match)
+	(progn
+	  (vm-folders-summary-compile-format format)
+	  (setq match
+		(assoc format vm-folders-summary-compiled-format-alist))))
+    ;; The local variable name `vm-folder-summary' is mandatory here for
+    ;; the format s-expression to work.
+    (let ((vm-folder-summary layout))
+      (eval (cdr match)))))
+
+(defun vm-folders-summary-compile-format (format)
+  (let ((return-value (vm-folders-summary-compile-format-1 format 0)))
+    (setq vm-folders-summary-compiled-format-alist
+	  (cons (cons format (nth 1 return-value))
+		vm-folders-summary-compiled-format-alist))))
+
+(defun vm-folders-summary-compile-format-1 (format start-index)
+  (let ((case-fold-search nil)
+	(done nil)
+	(sexp nil)
+	(sexp-fmt nil)
+	(last-match-end start-index)
+	new-match-end conv-spec)
+    (store-match-data nil)
+    (while (not done)
+      (while
+	  (and (not done)
+	       (string-match
+		"%\\(-\\)?\\([0-9]+\\)?\\(\\.\\(-?[0-9]+\\)\\)?\\([()dfnstu%]\\)"
+		format last-match-end))
+	(setq conv-spec (aref format (match-beginning 5)))
+	(setq new-match-end (match-end 0))
+	(if (memq conv-spec '(?\( ?d ?f ?n ?s ?t ?u))
+	    (progn
+	      (cond ((= conv-spec ?\()
+		     (save-match-data
+		       (let ((retval
+			      (vm-folder-summary-compile-format-1
+			       format
+			       (match-end 5))))
+			 (setq sexp (cons (nth 1 retval) sexp)
+			       new-match-end (car retval)))))
+		    ((= conv-spec ?d)
+		     (setq sexp (cons (list 'vm-fs-deleted-count-of
+					    'vm-folder-summary) sexp)))
+		    ((= conv-spec ?f)
+		     (setq sexp (cons (list 'vm-fs-short-folder-of
+					    'vm-folder-summary) sexp)))
+		    ((= conv-spec ?n)
+		     (setq sexp (cons (list 'vm-fs-new-count-of
+					    'vm-folder-summary) sexp)))
+		    ((= conv-spec ?t)
+		     (setq sexp (cons (list 'vm-fs-total-count-of
+					    'vm-folder-summary) sexp)))
+		    ((= conv-spec ?s)
+		     (setq sexp (cons (list 'vm-fs-spooled
+					    'vm-folder-summary) sexp)))
+		    ((= conv-spec ?u)
+		     (setq sexp (cons (list 'vm-fs-unread-count-of
+					    'vm-folder-summary) sexp))))
+	      (cond ((and (match-beginning 1) (match-beginning 2))
+		     (setcar sexp
+			     (list
+			      (if (eq (aref format (match-beginning 2)) ?0)
+				  'vm-numeric-left-justify-string
+				'vm-left-justify-string)
+			      (car sexp)
+			      (string-to-int
+			       (substring format
+					  (match-beginning 2)
+					  (match-end 2))))))
+		    ((match-beginning 2)
+		     (setcar sexp
+			     (list
+			      (if (eq (aref format (match-beginning 2)) ?0)
+				  'vm-numeric-right-justify-string
+				'vm-right-justify-string)
+			      (car sexp)
+			      (string-to-int
+			       (substring format
+					  (match-beginning 2)
+					  (match-end 2)))))))
+	      (cond ((match-beginning 3)
+		     (setcar sexp
+			     (list 'vm-truncate-string (car sexp)
+				   (string-to-int
+				    (substring format
+					       (match-beginning 4)
+					       (match-end 4)))))))
+	      (setq sexp-fmt
+		    (cons "%s"
+			  (cons (substring format
+					   last-match-end
+					   (match-beginning 0))
+				sexp-fmt))))
+	  (setq sexp-fmt
+		(cons (if (eq conv-spec ?\))
+			  (prog1 "" (setq done t))
+			"%%")
+		      (cons (substring format
+				       (or last-match-end 0)
+				       (match-beginning 0))
+			    sexp-fmt))))
+	(setq last-match-end new-match-end))
+      (if (not done)
+	  (setq sexp-fmt
+		(cons (substring format last-match-end (length format))
+		      sexp-fmt)
+		done t))
+      (setq sexp-fmt (apply 'concat (nreverse sexp-fmt)))
+      (if sexp
+	  (setq sexp (cons 'format (cons sexp-fmt (nreverse sexp))))
+	(setq sexp sexp-fmt)))
+    (list last-match-end sexp)))
+
+(defun vm-update-folders-summary-entry (fs)
+  (if (and (vm-fs-start-of fs)
+	   (marker-buffer (vm-fs-start-of fs)))
+      (let ((modified (buffer-modified-p))
+	    (do-mouse-track
+	     (and vm-mouse-track-summary
+		  (vm-mouse-support-possible-p)))
+	    summary)
+	(save-excursion
+	  (set-buffer (marker-buffer (vm-fs-start-of fs)))
+	  (let ((buffer-read-only nil))
+	    (unwind-protect
+		(save-excursion
+		  (goto-char (vm-fs-start-of fs))
+		  ;; We do a little dance to update the text in
+		  ;; order to make the markers in the text do
+		  ;; what we want.
+		  ;;
+		  ;; 1. We need to avoid having the start
+		  ;;    and end markers clumping together at
+		  ;;    the start position.
+		  ;;
+		  ;; 2. We want the window point marker (w->pointm
+		  ;;    in the Emacs display code) to move to the
+		  ;;    start of the summary entry if it is
+		  ;;    anywhere within the su-start-of to
+		  ;;    su-end-of region.
+		  ;;
+		  ;; We achieve (2) by deleting before inserting.
+		  ;; Reversing the order of insertion/deletion
+		  ;; pushes the point marker into the next
+		  ;; summary entry. We achieve (1) by inserting a
+		  ;; placeholder character at the end of the
+		  ;; summary entry before deleting the region.
+		  (goto-char (vm-fs-end-of fs))
+		  (insert-before-markers "z")
+		  (goto-char (vm-fs-start-of fs))
+		  (delete-region (point) (1- (vm-fs-end-of fs)))
+		  (insert
+		   (vm-folders-summary-sprintf vm-folders-summary-format fs))
+		  (delete-char 1)
+		  (and do-mouse-track
+		       (vm-mouse-set-mouse-track-highlight
+			(vm-fs-start-of fs)
+			(vm-fs-end-of fs)
+			(vm-fs-mouse-track-overlay-of fs))))
+	      (set-buffer-modified-p modified)))))))
+
+(defun vm-folders-summary-mode-internal ()
+  (setq mode-name "VM Folders Summary"
+	major-mode 'vm-folders-summary-mode
+	mode-line-format '("     %b")
+	;; must come after the setting of major-mode
+	mode-popup-menu (and vm-use-menus vm-popup-menu-on-mouse-3
+			     (vm-menu-support-possible-p)
+			     (vm-menu-mode-menu))
+	buffer-read-only t
+	buffer-offer-save nil
+	truncate-lines t)
+  (and vm-xemacs-p (featurep 'scrollbar)
+       (set-specifier scrollbar-height (cons (current-buffer) 0)))
+  (use-local-map vm-folders-summary-mode-map)
+  (and (vm-menu-support-possible-p)
+       (vm-menu-install-menus))
+  (if (and vm-mutable-frames vm-frame-per-folders-summary)
+      (vm-set-hooks-for-frame-deletion))
+  (run-hooks 'vm-folders-summary-mode-hook))
+
+(defun vm-do-folders-summary ()
+  (catch 'done
+    (let ((fs-hash (make-vector 89 0)) db dp fp f key fs totals
+          (format vm-folders-summary-format)
+	  (do-mouse-track (and vm-mouse-track-summary
+			       (vm-mouse-support-possible-p))))
+      (save-excursion
+	(set-buffer vm-folders-summary-buffer)
+	(erase-buffer)
+	(let ((buffer-read-only nil))
+	  (if (null vm-folders-summary-database)
+	      (throw 'done nil))
+	  (if (not (featurep 'berkeley-db))
+	      (throw 'done nil))
+	  (if (null (setq db (vm-open-folders-summary-database "r")))
+	      (throw 'done nil))
+	  (setq dp vm-folders-summary-directories)
+	  (while dp
+	    (if (cdr vm-folders-summary-directories)
+		(insert (car dp) ":\n"))
+	    (setq fp (sort (vm-delete-backup-file-names
+			    (vm-delete-auto-save-file-names
+			     (vm-delete-index-file-names
+			      (vm-delete-directory-names
+			       (directory-files (car dp))))))
+			   (function string-lessp)))
+	    (while fp
+	      (setq f (car fp)
+		    key (vm-make-folders-summary-key f (car dp))
+		    totals (get-database key db))
+	      (if (null totals)
+		  (let ((ff (expand-file-name f (car dp))))
+		    (setq totals (list (or (vm-count-messages-in-file ff) -1)
+				       -1 -1 -1))
+		    (if (eq (car totals) -1)
+			nil
+		      (vm-store-folder-totals ff totals)))
+		(setq totals (read totals)))
+	      (if (eq (car totals) -1)
+		  nil
+		(setq fs (vm-make-folder-summary))
+		(vm-set-fs-folder-of fs (expand-file-name f (car dp)))
+		(vm-set-fs-short-folder-of fs f)
+		(vm-set-fs-total-count-of fs (vm-nonneg-string (car totals)))
+		(vm-set-fs-new-count-of fs (vm-nonneg-string (nth 1 totals)))
+		(vm-set-fs-unread-count-of fs (vm-nonneg-string
+					       (nth 2 totals)))
+		(vm-set-fs-deleted-count-of fs (vm-nonneg-string
+						(nth 3 totals)))
+		(vm-set-fs-folder-key-of fs key)
+		(vm-set-fs-start-of fs (vm-marker (point)))
+		(insert (vm-folders-summary-sprintf format fs))
+		(vm-set-fs-end-of fs (vm-marker (point)))
+		(and do-mouse-track
+		     (vm-set-fs-mouse-track-overlay-of
+		      fs
+		      (vm-mouse-set-mouse-track-highlight
+		       (vm-fs-start-of fs)
+		       (vm-fs-end-of fs))))
+		(set (intern key fs-hash) fs))
+	      (setq fp (cdr fp)))
+	    (setq dp (cdr dp)))
+	  (close-database db)
+	  (setq vm-folders-summary-hash fs-hash))
+	(goto-char (point-min))))))
+
+(defun vm-update-folders-summary-highlight ()
+  (if (or (null vm-mail-buffer)
+	  (null (buffer-file-name vm-mail-buffer))
+	  (null vm-folders-summary-hash))
+      (progn
+	(and vm-folders-summary-overlay
+	     (vm-set-extent-endpoints vm-folders-summary-overlay 1 1))
+	(setq vm-mail-buffer nil))
+    (let ((ooo vm-folders-summary-overlay)
+	  (fs (symbol-value (intern-soft (vm-make-folders-summary-key
+					  (buffer-file-name vm-mail-buffer))
+					 vm-folders-summary-hash))))
+      (if (and fs
+	       (or (null ooo)
+		   (null (vm-extent-object ooo))
+		   (/= (vm-extent-end-position ooo)
+		       (vm-fs-end-of fs))))
+	  (vm-folders-summary-highlight-region
+	   (vm-fs-start-of fs) (vm-fs-end-of fs)
+	   vm-summary-highlight-face)))))
+
+(defun vm-do-needed-folders-summary-update ()
+  (if (null vm-folders-summary-buffer)
+      nil
+    (save-excursion
+      (set-buffer vm-folders-summary-buffer)
+      (if (or (eq vm-modification-counter vm-flushed-modification-counter)
+	      (null vm-folders-summary-hash))
+	  nil
+	(mapatoms
+	 (function
+	  (lambda (sym)
+	    (let ((fs (symbol-value sym)))
+	      (if (null (vm-fs-modflag-of fs))
+		  nil
+		(vm-update-folders-summary-entry fs)
+		(vm-set-fs-modflag-of fs nil)))))
+	  vm-folders-summary-hash)
+	(vm-update-folders-summary-highlight)
+	(setq vm-flushed-modification-counter vm-modification-counter)))))
+
+(defun vm-mark-for-folders-summary-update (folder &optional dont-descend)
+  (let ((key (vm-make-folders-summary-key folder))
+	(hash vm-folders-summary-hash)
+	(spool-hash vm-folders-summary-spool-hash)
+	list fs )
+    (setq fs (symbol-value (intern-soft key hash)))
+    (if (not fs)
+	nil
+      (vm-set-fs-modflag-of fs t)
+      (vm-check-for-killed-summary)
+      (if vm-folders-summary-buffer
+	  (save-excursion
+	    (set-buffer vm-folders-summary-buffer)
+	    (vm-increment vm-modification-counter))))
+    (if dont-descend
+	nil
+      (setq list (symbol-value (intern-soft key spool-hash)))
+      (while list
+	(vm-mark-for-folders-summary-update (car list) t)
+	(setq list (cdr list))))))
+
+(defun vm-make-folders-summary-associative-hashes ()
+  (let ((triples (vm-compute-spool-files t))
+	(spool-hash (make-vector 61 0))
+	(folder-hash (make-vector 61 0))
+	s-list f-list folder-key spool-key)
+    (while triples
+      (setq folder-key (vm-make-folders-summary-key (car (car triples)))
+	    spool-key (vm-make-folders-summary-key (nth 1 (car triples)))
+	    s-list (symbol-value (intern-soft spool-key spool-hash))
+	    s-list (cons (car (car triples)) s-list)
+	    f-list (symbol-value (intern-soft folder-key folder-hash))
+	    f-list (cons (nth 1 (car triples)) f-list)
+	    triples (cdr triples))
+      (set (intern spool-key spool-hash) s-list)
+      (set (intern folder-key folder-hash) f-list))
+    (setq vm-folders-summary-spool-hash spool-hash)
+    (setq vm-folders-summary-folder-hash folder-hash)))
+
+(defun vm-follow-folders-summary-cursor ()
+  (if (or (not (eq major-mode 'vm-folders-summary-mode))
+	  (null vm-folders-summary-hash))
+      nil
+    (catch 'done
+      (mapatoms
+       (function
+	(lambda (sym)
+	  (let ((fs (symbol-value sym)))
+	    (if (and (>= (point) (vm-fs-start-of fs))
+		     (< (point) (vm-fs-end-of fs))
+		     (or (null vm-mail-buffer)
+			 (not (eq vm-mail-buffer
+				  (vm-get-file-buffer (vm-fs-folder-of fs))))))
+		(progn
+		  (setq vm-mail-buffer
+			(save-excursion
+			  (vm-visit-folder (vm-fs-folder-of fs))
+			  (current-buffer)))
+		  (vm-increment vm-modification-counter)
+		  (vm-update-summary-and-mode-line)
+		  (throw 'done t))))))
+       vm-folders-summary-hash)
+      nil )))
