@@ -2,7 +2,7 @@
 
 (provide 'vm-version)
 
-(defconst vm-version "6.75"
+(defconst vm-version "6.89"
   "Version number of VM.")
 
 (defun vm-version ()
@@ -11,10 +11,12 @@
 
 (defconst vm-xemacs-p nil)
 (defconst vm-xemacs-mule-p nil)
+(defconst vm-xemacs-file-coding-p nil)
 (defconst vm-fsfemacs-p nil)
 (defconst vm-fsfemacs-mule-p nil)
 (defun vm-xemacs-p () vm-xemacs-p)
 (defun vm-xemacs-mule-p () vm-xemacs-mule-p)
+(defun vm-xemacs-file-coding-p () vm-xemacs-file-coding-p)
 (defun vm-fsfemacs-p () vm-fsfemacs-p)
 (defun vm-fsfemacs-mule-p () vm-fsfemacs-mule-p)
 (defun vm-note-emacs-version ()
@@ -22,6 +24,9 @@
 	vm-xemacs-mule-p (and vm-xemacs-p (featurep 'mule)
 			      ;; paranoia
 			      (fboundp 'set-buffer-file-coding-system))
+	vm-xemacs-file-coding-p (and vm-xemacs-p (featurep 'file-coding)
+				     ;; paranoia
+				     (fboundp 'set-buffer-file-coding-system))
 	vm-fsfemacs-p (not vm-xemacs-p)
 	vm-fsfemacs-mule-p (and (not vm-xemacs-mule-p) (featurep 'mule)
 				(fboundp 'set-buffer-file-coding-system))))
@@ -39,6 +44,11 @@
   (and vm-fsfemacs-p
        (fboundp 'menu-bar-mode)))
 
+(defun vm-menu-fsfemacs19-menus-p ()
+  (and vm-fsfemacs-p
+       (fboundp 'menu-bar-mode)
+       (= emacs-major-version 19)))
+
 (defun vm-menu-xemacs-menus-p ()
   (and vm-xemacs-p
        (fboundp 'set-buffer-menubar)))
@@ -69,7 +79,8 @@
 	 (fboundp 'menu-bar-mode))))
  
 (defun vm-toolbar-support-possible-p ()
-  (and vm-xemacs-p (featurep 'toolbar)))
+  (or (and vm-xemacs-p (featurep 'toolbar))
+      (and vm-fsfemacs-p (fboundp 'tool-bar-mode))))
 
 (defun vm-multiple-fonts-possible-p ()
   (cond (vm-xemacs-p
@@ -78,5 +89,10 @@
 	 (memq window-system '(x w32 win32)))))
 
 (defun vm-images-possible-here-p ()
-  (and vm-xemacs-p (memq (device-type) '(x mswindows))))
+  (or (and vm-xemacs-p (memq (device-type) '(x mswindows)))
+      (and vm-fsfemacs-p window-system (fboundp 'image-type-available-p))))
 
+(defun vm-image-type-available-p (type)
+  (if (fboundp 'image-type-available-p)
+      (image-type-available-p type)
+    (featurep type)))
